$body-background-color: #1e1e1e;
$sidebar-color: #27272a;
$body-text-color: #d4d4d8;
$body-heading-color: #e4e4e7;
$link-color: #60a5fa;
$nav-child-link-color: #a1a1aa;
$search-background-color: #27272a;
$search-result-preview-color: #a1a1aa;
$border-color: #3f3f46;
$code-background-color: #2a2a2e;
$feedback-color: #a1a1aa;
$table-background-color: #27272a;
$btn-primary-color: #60a5fa;
